##############################################################################
#
# License
# -------
#
# Copyright (c) Gerard Hale. All right reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#
# 3. Gerard Hale requests that attribution be given to Zope
#    in any manner possible. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
#
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
#
#      "This product includes software developed by Gerard Hale
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
#
# 5. Names associated with Gerard Hale must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Gerard Hale.
#
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
#
#      "This product includes software developed by Gerard Hale
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
#
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official software releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.
#
#
# Disclaimer
#
#   THIS SOFTWARE IS PROVIDED BY Gerard Hale ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL Gerard Hale OR HIS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
#
#
# This software consists of contributions made by Gerard Hale and
# many other individuals.
#
##############################################################################
__doc__="""ZSession Product"""
__version__='0.0.9.5'

import ZSession, threading
from Globals import ImageFile
from ZSession import logProcess

"""
We only want one instance of the ZSession class,
therefore we're not using registerClass
Because no registerClass, use the old method to
set the icon for the ZSession object
"""

misc_ = { 'ZSession_icon': ImageFile('www/ZSession_icon.gif', globals()) }

def initialize(context):
    """
    Initialize the ZSession Product.
    The ZSession exists in the root folder so it
    can be accessed all objects in Zope.
    """

    try:
        """Add ZSession to the Zope root"""
        app = context._ProductContext__app

        global ZSession
        if hasattr(app, 'ZSession'):
            ZSession = getattr(app, 'ZSession', None)
        else:
            try:
                """Creating Session"""
                oZSession=ZSession.ZSession('_ZSession', 20)
                app._setObject('ZSession', oZSession)
                get_transaction().note('Added ZSession')
                get_transaction().commit()
            except:
                """failed to Add ZSession"""
                pass
            ZSession = getattr(app, 'ZSession', None)

        """Raises exception in startup if it does not exists.
           This is a fatal Error - No ZSession!"""
        assert ZSession is not None

        blnThreadFound = 0
        for t in threading.enumerate():
            if (t.getName() == "ZSession"):
                blnThreadFound = 1
        if(blnThreadFound==0):
            cleanup=threading.Thread(target=ZSession.ZSession.clearSessionsProcess, name='ZSession', args=())
            """So thread will terminate with parent process."""
            cleanup.setDaemon(1)
            cleanup.start()

    except:
        """Print Tracback to console to alert any problems"""
        import traceback, sys
        traceback.print_exc()
        logProcess("initialize error: %s" % (sys.exc_info()))

    """Register Help Files in Help Directory to show in Zope Help System"""
    context.registerHelp()

    logProcess("ZSession Initialized")
